document.addEventListener('DOMContentLoaded', function (event) {
  var baseHost = document.location.origin
  var streamUrl = baseHost + ':81'
  const view = document.getElementById('stream')
  const viewContainer = document.getElementById('stream-container')
  const streamButton = document.getElementById('toggle-stream')
  const saveButton = document.getElementById('save-still')
  const sdcButton = document.getElementById('save-sd')
  const data_status = document.getElementById('data-status')
  const data_time = document.getElementById('data-time')
  const prev = document.getElementById('prev')
  const next = document.getElementById('next')
  const ref1 = document.getElementById('href1');
  const ref2 = document.getElementById('href2');
  const ref3 = document.getElementById('href3');
  const ref4 = document.getElementById('href4');
  let graph = document.getElementsByTagName('canvas')[0].getContext('2d');
  let filename;

  let gow = graph.canvas.width;
  let goh = graph.canvas.height;
  let giw = 24*30;
  let gih = goh - 30;
  let gix = 40;
  let giy = 10;
  let histroy_day = 0;

  let sdc_flag      = 0;
  let history_flag  = 0;
  let config_flag   = 0;
  let hist_date_flag = 0;
  let hist_date     = "";
  let ref1_date     = "";
  let ref2_date     = "";
  let ref3_date     = "";
  let ref4_date     = "";
  
  function fetchUrl(url, cb){
    fetch(url)
      .then(function (response) {
        if (response.status !== 200) {
          cb(response.status, response.statusText);
        } else {
          response.text().then(function(data){
            cb(200, data);
          }).catch(function(err) {
            cb(-1, err);
          });
        }
      })
      .catch(function(err) {
        cb(-1, err);
      });
  }
  const hide = el => { el.classList.add('hidden') }
  const show = el => { el.classList.remove('hidden') }
  const disable = el => { el.classList.add('disabled'); el.disabled = true;  }
  const enable = el => {  el.classList.remove('disabled'); el.disabled = false; }
  const updateValue = (el, value, updateRemote) => {
    updateRemote = updateRemote == null ? true : updateRemote
    let initialValue
    if (el.type === 'checkbox') {
      initialValue = el.checked
      value = !!value
      el.checked = value
    } else {
      initialValue = el.value
      el.value = value
    }
    if (updateRemote && initialValue !== value) {
      updateConfig(el);
    } else if(!updateRemote){
    }
  }

  function updateConfig (el) {
    let value
    switch (el.type) {
      case 'checkbox':
        value = el.checked ? 1 : 0
        break
      case 'range':
      case 'select-one':
        value = el.value
        break
      case 'button':
      case 'submit':
        value = '1'
        break
      default:
        return
    }

    const query = `${baseHost}/control?var=${el.id}&val=${value}`

    fetch(query)
      .then(response => {
        console.log(`request to ${query} finished, status: ${response.status}`)
      })
  }
  // read initial values
  fetch(`${baseHost}/status`)
    .then(function (response) {
      return response.json()
    })
    .then(function (state) {
      document
        .querySelectorAll('.default-action')
        .forEach(el => {
          updateValue(el, state[el.id], false)
        })
    })

  function dispData( data){
    data_status.innerText = "T: " +data.t +"C   "+ " H: "+ data.h + "%%RH ";    data_time.innerHTML = data.tm;
    setTimeout(() => { getData(); }, 500);
  }

  function getData( ){
    if( sdc_flag){ sdc_flag = 0;  filename = "./sdc.json"; fetch( filename) .then( response => response.json()) .then( data => dispHistory( data) ) .catch((reason) => { console.log( reason); });
    } else if( config_flag){ config_flag = 0; filename = "./config.json?s=0,"; fetch( filename) .then( response => response.json()) .then( data => dispRef( data) ) .catch((reason) => { console.log( reason); });
    } else if( history_flag){ history_flag = 0; filename = "./history.json?s="+histroy_day; fetch( filename) .then( response => response.json()) .then( data => dispHistory( data) ) .catch((reason) => { console.log( reason); });
    } else if( hist_date_flag){ hist_date_flag = 0; filename = "./hist_date.json?s="+hist_date; fetch( filename) .then( response => response.json()) .then( data => dispHistory( data) ) .catch((reason) => { console.log( reason); });
    } else {   fetch("./data.json") .then( response => response.json()) .then( data => dispData( data) )  .catch((reason) => { console.log( reason); });
    }
  }

  function getConfig( ){  config_flag = 1;}
  function getHistory( ){ history_flag = 1; }
  function getHistoryDate( ){ if( hist_date != ""){ hist_date_flag = 1; } }
  function set_sdc( ) {   sdc_flag = 1;  }

  const stopStream = () => {
    window.stop();
    streamButton.innerHTML = 'Start Stream'
    view.src = `${baseHost}/capture?_cb=${Date.now()}`
    show(viewContainer)
  }

  const startStream = () => {
    view.src = `${streamUrl}/stream`
    show(viewContainer)
    streamButton.innerHTML = 'Stop Stream'
  }

  // Attach actions to buttons

  streamButton.onclick = () => {
    const streamEnabled = streamButton.innerHTML === 'Stop Stream'
    if (streamEnabled) {
      stopStream()
    } else {
      startStream()
    }
  }
  
  saveButton.onclick = () => {
    var canvas = document.createElement("canvas");
    canvas.width = view.width;
    canvas.height = view.height;
    document.body.appendChild(canvas);
    var context = canvas.getContext('2d');
    context.drawImage(view,0,0);
    try {
      var dataURL = canvas.toDataURL('image/jpeg');
      saveButton.href = dataURL;
      var d = new Date();
      saveButton.download = d.getFullYear() + ("0"+(d.getMonth()+1)).slice(-2) + ("0" + d.getDate()).slice(-2) + ("0" + d.getHours()).slice(-2) + ("0" + d.getMinutes()).slice(-2) + ("0" + d.getSeconds()).slice(-2) + ".jpg";
    } catch (e) {
      console.error(e);
    }
    canvas.parentNode.removeChild(canvas);
  }

  sdcButton.onclick = () =>{ if( window.confirm( "save SDC")){ sdc_flag = 1; } }
  prev.onclick = () =>{      if( histroy_day > -10){ histroy_day = histroy_day - 1; getHistory(); } }
  next.onclick = () =>{      if( histroy_day < 0){   histroy_day = histroy_day + 1; getHistory(); } }
  ref1.onclick = () =>{      hist_date = ref1_date;  getHistoryDate(); }
  ref2.onclick = () =>{      hist_date = ref2_date;  getHistoryDate(); }
  ref3.onclick = () =>{      hist_date = ref3_date;  getHistoryDate(); }
  ref4.onclick = () =>{      hist_date = ref4_date;  getHistoryDate(); }
  
  function pregraph() {
    let i;

    graph.fillStyle = 'rgba(255,255,192,1)';    graph.fillRect( 0, 0, gow, goh);
    graph.fillStyle = 'rgba(192,192,192,1)';    for(i = 0 ; i <= giw; i += 30){ graph.fillRect( gix+i, giy, 1, gih); }
                                                for(i = 0 ; i <= gih; i += 20){ graph.fillRect( gix, giy+i, giw, 1);  }
    graph.fillStyle = 'rgba(192,192,192,1)';    for(i = 0 ; i <= giw; i += 30){ graph.fillText( (i/30).toString(), gix+i-5, giy+gih+10); }
                                                for(i = 0 ; i <= gih; i += 20){ graph.fillText( (i/2-20).toString(), gix-20, giy+gih-i+5); }
    graph.stroke();
    graph.restore();
  }

  function dispHistory( hist){
    let i;
    let h;
    let d;
    
    pregraph();
    if( hist.n){
      graph.fillStyle = 'rgba(255,0,0,1)';
      for(i = 0 ; i < hist.n; i++){
        d = hist.t[ i]*gih/120+40;
        if( d < gih){
          h = d;
        } else {
          h = gih;
        }
        if( d != 20){
          graph.fillRect( gix+i/2, giy+gih-h, 1, 1);
        }
      }
      graph.fillStyle = 'rgba(0,0,255,1)';
      for(i = 0 ; i < hist.n; i++){
        d = hist.h[ i]*gih/120+40;
        if( d < gih){
          h = d;
        } else {
          h = gih;
        }
        if( d != 20){
          graph.fillRect( gix+i/2, giy+gih-h, 1, 1);
        }
      }
      graph.fillStyle = 'rgba(128,128,128,1)';
      graph.fillText( hist.x, gix+10, giy+10);
      graph.stroke();
      graph.restore();
      if( histroy_day == 0){
        prev.innerText = "prev";
        next.innerText = "----";
      } else {
          prev.innerText = "prev";
          next.innerText = "next";
        }
    } else {
      prev.innerText = "---";
      next.innerText = "---";
    }
    setTimeout(() => { getData(); }, 500);
  }
  function dispRef( data){
    if( data.ref1 != ""){ ref1.innerText = data.ref1}
    if( data.ref2 != ""){ ref2.innerText = data.ref2}
    if( data.ref3 != ""){ ref3.innerText = data.ref3}
    if( data.ref4 != ""){ ref4.innerText = data.ref4}
    ref1_date = data.ref1;
    ref2_date = data.ref2;
    ref3_date = data.ref3;
    ref4_date = data.ref4;
    setTimeout(() => { getData(); }, 500);
  }  

  // Attach default on change action
  document
    .querySelectorAll('.default-action')
    .forEach(el => {
      el.onchange = () => updateConfig(el)
    })

  getConfig();
  getHistory();
  getData();
  startStream();
})
